#' @title Summarize multinomial models
#'
#' @description This function summarizes multinomial models computed with nnet::multinom.
#'
#' @param some.multinom.model A nnet object created by a call to nnet::multinom
#'
#' @examples
#' require(nnet); set.seed(1)
#' RESP <- gl(3, 100)
#' PRED <- 1:300
#' some.multinom.model <- multinom(RESP ~ PRED)
#' summarize.multinom(some.multinom.model)
#'
#' @return A list with 5 components: (i) a data frame with model coefficients, (ii) a confusion matrix, (iii) a numeric vector with classification metrics, (iv) a numeric vector with deviances and R-squareds, (v) an overall model significance test
#'
#' @export
#'
#' @author Stefan Th. Gries (www: <https://www.stgries.info>, ORCID id: 0000-0002-6497-3958)

summarize.multinom <- function (some.multinom.model) {
   response <- model.frame(some.multinom.model)[,1]
   null.model <- update(some.multinom.model, ~1)
   some.models.lrt <- anova(null.model, some.multinom.model, test="Chisq")[2,]
   some.models.summary <- summary(some.multinom.model)
   some.models.zscores <- some.models.summary$coefficients / some.models.summary$standard.errors
   some.models.pvalues <- 2*pnorm(abs(some.models.zscores), lower.tail=FALSE)

   output <- list(); output[["Coefficients"]] <- data.frame(
      "Estimate"=as.numeric(some.models.summary$coefficients),
      "Std.Error"=as.numeric(some.models.summary$standard.errors),
      "z"=as.numeric(some.models.zscores),
      "p2tailed"=as.numeric(some.models.pvalues))
      output[["Coefficients"]] <- cbind(output[["Coefficients"]],
         "2.5%" =output[["Coefficients"]][,1] - (qnorm(0.975)*output[["Coefficients"]][,2]),
         "97.5%"=output[["Coefficients"]][,1] + (qnorm(0.975)*output[["Coefficients"]][,2]))
      output[["Coefficients"]] <- output[["Coefficients"]][,c(1:2,5:6,3:4)]
      if ("matrix" %in% class(some.models.summary$coefficients)) {
         row.names(output[["Coefficients"]]) <- paste(
            rep(colnames(some.models.summary$coefficients), each=nrow(some.models.summary$coefficients)),
            rep(rownames(some.models.summary$coefficients),      ncol(some.models.summary$coefficients)),
            sep=":")
      } else {
         row.names(output[["Coefficients"]]) <- names(some.models.summary$coefficients)
      }
   predictions <- predict(some.multinom.model)
   output[["Confusion matrix"]] <- table(
      OBS =response,
      PRED=predictions)
   output[["Classification metrics"]] <- c(
      "Baseline"=max(prop.table(table(response))),
      "Accuracies"=c(tapply( # accuracies for each level & overall
         response==predictions,
         response, mean),
         "overall"=mean(response==predictions)))
      names(output[["Classification metrics"]]) <- gsub("Accuracies\\.", "Acc. for ", names(output[["Classification metrics"]]), perl=TRUE)
      if (tail(output[["Classification metrics"]], 1) - head(output[["Classification metrics"]], 1) > 0) {
         output[["Classification metrics"]]["pbinom"] <- sum(dbinom(
            sum(predictions==response):nrow(model.frame(some.multinom.model)),
            nrow(model.frame(some.multinom.model)),
            output[["Classification metrics"]][1]))
      } else {
         output[["Classification metrics"]]["pbinom"] <- NA
      }
      output[["Classification metrics"]]["logloss"] <- deviance(some.multinom.model) / (2*nrow(model.frame(some.multinom.model)))
   output[["Deviances & R2s"]] <- c(
      "Deviance of null model"=deviance(null.model),
      "Deviance of this model"=deviance(some.multinom.model),
      "McFadden R2"=(deviance(null.model)-deviance(some.multinom.model)) / deviance(null.model),
      "Nagelkerke R2"=as.numeric((1 - exp((-2*logLik(some.multinom.model) - -2*logLik(null.model))/nrow(model.frame(some.multinom.model)))) / (1 - exp(2*logLik(null.model)/nrow(model.frame(some.multinom.model))))))
   output[["Model significance test"]] <- c(
      "LR statistic"=some.models.lrt[1,6]  , "Df"=some.models.lrt[1,5],
      "P-value"=pchisq(some.models.lrt[1,6], some.models.lrt[1,5], lower.tail=FALSE),
      "Number of data points"=nrow(model.frame(some.multinom.model)))
   return(output)
}
