#' @title Summarize ordinal models
#'
#' @description This function summarizes ordinal models computed with MASS::polr
#'
#' @param some.polr.model A polr object created by a call to MASS::polr
#'
#' @examples
#' require(MASS); set.seed(1)
#' RESP <- gl(3, 100, ordered=TRUE)
#' PRED <- sample(1:300)
#' some.polr.model <- polr(RESP ~ PRED)
#' summarize.polr(some.polr.model)
#'
#' @return A list with 5 components: (i) a data frame with model coefficients, (ii) a confusion matrix, (iii) a numeric vector with classification metrics, (iv) a numeric vector with deviances and R-squareds, (v) an overall model significance test
#'
#' @export
#'
#' @author Stefan Th. Gries (www: <https://www.stgries.info>, ORCID id: 0000-0002-6497-3958)

summarize.polr <- function (some.polr.model) {
   null.model <- update(some.polr.model, ~1)
   some.models.lrt <- anova(null.model, some.polr.model, test="Chisq")[2,]
   some.models.summary <- summary(some.polr.model)
   some.models.zscores <- some.models.summary$coefficients[,3]
   some.models.pvalues <- 2*pnorm(abs(some.models.zscores), lower.tail=FALSE)

   output <- list(); output[["Coefficients"]] <- data.frame(
      "Estimate"=as.numeric(some.models.summary$coefficients[,1]),
      "Std.Error"=as.numeric(some.models.summary$coefficients[,2]),
      "z"=as.numeric(some.models.zscores),
      "p2tailed"=as.numeric(some.models.pvalues))
      output[["Coefficients"]] <- cbind(output[["Coefficients"]],
         "2.5%" =output[["Coefficients"]][,1] - (qnorm(0.975)*output[["Coefficients"]][,2]),
         "97.5%"=output[["Coefficients"]][,1] + (qnorm(0.975)*output[["Coefficients"]][,2]))
      output[["Coefficients"]] <- output[["Coefficients"]][,c(1:2,5:6,3:4)]
      rownames(output[["Coefficients"]]) <- names(some.models.zscores)
   output[["Coefficients"]] <-
      output[["Coefficients"]][c(
         length(some.polr.model$zeta):nrow(output$Coefficients),
         1:length(some.polr.model$coefficients)),]
   predictions <- predict(some.polr.model)
   output[["Confusion matrix"]] <- table(
      OBS =some.polr.model$model[,1],
      PRED=predictions)
   output[["Classification metrics"]] <- c(
      "Baseline"=max(prop.table(table(some.polr.model$model[,1]))),
      "Accuracies"=c(tapply( # accuracies for each level & overall
         as.character(some.polr.model$model[,1])==as.character(predictions),
         some.polr.model$model[,1], mean),
         "overall"=mean(as.character(some.polr.model$model[,1])==as.character(predictions))))
      names(output[["Classification metrics"]]) <- gsub("Accuracies\\.", "Acc. for ", names(output[["Classification metrics"]]), perl=TRUE)
      output[["Classification metrics"]]["logloss"] <- deviance(some.polr.model) / (2 * nrow(model.frame(some.polr.model)))
   output[["R2s"]] <- c(
      "Deviance of null model"=deviance(null.model),
      "Deviance of this model"=deviance(some.polr.model),
      "McFadden R2"=(deviance(null.model)-deviance(some.polr.model)) / deviance(null.model),
      "Nagelkerke R2"=as.numeric((1 - exp((-2*logLik(some.polr.model) - -2*logLik(null.model))/nrow(model.frame(some.polr.model)))) / (1 - exp(2*logLik(null.model)/nrow(model.frame(some.polr.model))))))
   output[["Model significance test"]] <- c(
      "LR statistic"=some.models.lrt[1,6], "Df"=some.models.lrt[1,5],
      "P-value"=pchisq(some.models.lrt[1,6], some.models.lrt[1,5], lower.tail=FALSE),
      "Number of data points"=nrow(model.frame(some.polr.model)))
   return(output)
}
